from parser import parse_miu
from cvx_gen import to_cvx
import argparse


def version():
    try:
        with open('../version.txt') as f:
            return f.read()
    except:
        pass       
    try:
        with open('version.txt') as f:
            return f.read() 
    except:
        pass
    try:
        with open('sydar/version.txt') as f:
            return f.read() 
    except:
        pass
def main():
    parser = argparse.ArgumentParser(description='Synthesis of Hybrid Systems.')
    parser.add_argument('input_file',type=str)
    parser.add_argument('-o','--output', help='Dumps the output to the specified file')
    parser.add_argument('-v','--version', action='version', version='sydar '+version())
    args = parser.parse_args()
    symbol_table = parse_miu(args.input_file)
    nodes = symbol_table.get_tagged_nodes()
    edges = symbol_table.get_tagged_edges()
    constants = symbol_table.container['system']
    output = to_cvx(nodes,edges,constants)

    if args.output is not None:
        with open(args.output, "w") as text_file:
            text_file.write(output)

    else:
        print output




"""
.. module:: helpers
   :platform: Unix
   :synopsis: 

.. moduleauthor:: Mohammed Alshiekh


"""

from region import *
from scipy import random

def generate_psd(size):
    matrixSize = size
    A = random.rand(matrixSize,matrixSize)
    psd = np.dot(A,A.transpose())
    return psd.tolist()

def generate_pd(size):
    psd = generate_psd(size)
    alpha = 0.1
    I = np.identity(size)
    pd = psd + alpha*I
    return pd.tolist()

def generate_point(size,shift=0):
    matrixSize = size
    point = random.rand(matrixSize,1)+shift
    return point[:,0].tolist()

def generate_ellipsoid(size,shift=0):
    pd = generate_pd(2)
    point = generate_point(2,shift)
    return Ellipsoid(pd,point)    

def generate_halfspace(size, shift=0):
    point = generate_point(2,shift)
    vector = generate_point(2,shift)
    return HalfSpace(point,vector)

def generate_canon_tree(n_regions=5, n_operations=2,regions=None):
    sub_tree1 = Tree('&',[x for x in regions[:2]])
    sub_tree2 = Tree('&',[x for x in regions[2:4]])
    sub_tree3 = Tree('&',[x for x in regions[4:6]])
    sub_trees = [sub_tree1, sub_tree2, sub_tree3]
    return Tree('||',sub_trees)



import numpy as np
import sys

def is_symmetric(matrix):
    return (matrix.transpose() == matrix).all()

def is_pos_def(matrix):
    return np.all(np.linalg.eigvals(matrix) > 0)

def is_pos_semidef(matrix):
    return np.all(np.linalg.eigvals(matrix) >= 0)

def is_point(coord):
    return (len(np.array(coord).shape) == 1) or np.array(coord).shape[0]==1 or np.array(coord).shape[0]==2

def is_PD(P):
    return isinstance(P, PDMatrix)

def is_center_point(c):
    return isinstance(c, CenterPoint)

def dim_check(P,c):
    return P.shape[0] == c.shape[0]

class Objective(object):
    def __init__(self, p, q, r):
        self.p = p
        self.q = q
        self.r = r
        
    def __str__(self):
        return 'Objective({}, {}, {})'.format(self.p, self.q, self.r)  

    def __repr__(self):
        return 'Objective({}, {}, {})'.format(self.p, self.q, self.r)  

class Matrix(object):
    def __init__(self, A):
        self.matrix = np.array(A)
        self.shape = self.matrix.shape
        
    @property
    def matrix(self):
        """I'm the 'matrix' property."""
        return self._matrix

    @matrix.setter
    def matrix(self, value):
        self._matrix = value
    
    def __str__(self):
        return '{}'.format(self.matrix)       

    @matrix.deleter
    def matrix(self):
        del self._matrix       
 
    def mmat(self, format='%.4e'):
        """Display the ndarray 'x' in a format suitable for pasting to MATLAB"""
        string = ''

        def print_row(row, format):
            sub_string = ''
            for i in row:
                sub_string += format % i + ' '
            return sub_string

        if self._matrix.ndim == 1:
            # 1d input
            string += "["
            string += print_row(self._matrix, format)
            string += "];\n"
            string += "\n"
            string = string[:-3]

        if self._matrix.ndim == 2:
            string += "["
            string += print_row(self._matrix[0], format)
            if self._matrix.shape[0] > 1:
                string += ';'
            for row in self._matrix[1:-1]:
                string += " "
                string += print_row(row, format)
                string += ";"
            if self._matrix.shape[0] > 1:
                string += " "
                string += print_row(self._matrix[-1], format)
            string += "];"
            string = string[:-1]

        if self._matrix.ndim > 2:
            d_to_loop = self._matrix.shape[2:]
            sls = [slice(None,None)]*2
            string += "reshape([ "
            # loop over flat index
            for i in range(prod(d_to_loop)):
                # reverse order for matlab
                # tricky double reversal to get first index to vary fastest
                ind_tuple = unravel_index(i,d_to_loop[::-1])[::-1]
                ind = sls + list(ind_tuple)
                string += mmat(self._matrix[ind],format)+"\n"         

            string += '],['
            for i in self._matrix.shape:
                string += '%d' % i
            string += '])'+"\n" 
        return string

class SymMatrix(Matrix):
    @Matrix.matrix.setter
    def matrix(self, value):
        if is_symmetric(value):
            self._matrix = value
        else:
            print "The matrix is not a symmetric matrix!"
            sys.exit(1)  

class PDMatrix(Matrix):
    @Matrix.matrix.setter
    def matrix(self, value):
        if is_pos_def(value):
            self._matrix = value
        else:
            print "The matrix is not a PD matrix!"
            sys.exit(1)          

class PSDMatrix(Matrix):
    @Matrix.matrix.setter
    def matrix(self, value):
        if is_pos_semidef(value):
            self._matrix = value
        else:
            print "The matrix is not a PSD matrix!"
            sys.exit(1)

class CenterPoint(object):
    def __init__(self, point):
        self.coord = np.array(point)
        self.shape = self.coord.shape
    @property
    def coord(self):
        """I'm the 'coord' property."""
        return self._coord

    @coord.setter
    def coord(self, value):
        if is_point(np.transpose(value)):
            self._coord = value
        else:
            print "The input must be a point, the given is {}".format(value)
            sys.exit(1)

    @coord.deleter
    def coord(self):
        del self._coord   

    def __str__(self):
        return '{}'.format(self.coord)                    
         
        
# def main():
#     c = CenterPoint([1,2,5])
#     h = HalfSpace([1,2,3],[4,3,8], 'h')
#     a = PDMatrix([[1,0,0],[0,1,0],[0,0,1]])
#     k = Matrix([[1,0,0],[0,1,0],[0,0,1]])
#     ellipsoid = Ellipsoid(a,c,'A')
#     print ellipsoid
#     print h
#     print a.mmat()

# if __name__ == '__main__':
#     main()


from region import *
from Automata import RegularLanguage
from sys import exit

def tag_nodes(graph, i=1):
    """takes in a graph and returns a dict of nodes
    """
    nodes = dict()
    history = dict()
    state_info_map = dict()
    for source, transitions in graph.iteritems():
        state_info_map[source] = list(transitions)[0][2:]
        for transition in list(transitions):
            if transition[1] not in nodes.keys():
                history[transition[1]] = [transition[0]]
            elif transition[0] not in history[transition[1]]:
                history[transition[1]].append(transition[0])
        for transition in list(transitions):
            nodes[transition[1]] = Tree('||', history[transition[1]])
    for key, value in nodes.iteritems():
        nodes[key] = {'obj':Objective('p_{}'.format(key[2:]),
                      'q_{}'.format(key[2:]),'r_{}'.format(key[2:])),
                      'region': value,
                      'accepting': state_info_map[key][0], 
                      'initial': state_info_map[key][1]}
    return nodes

def tag_edges(graph):
    """
    takes in a graph and returns a dict of edges
    """
    edges = dict()
    for source, transitions in graph.iteritems():
        for transition in list(transitions):
            edges[(source,transition[1])] = {'region': Tree('||', 
                                            [transition[0]])}
    return edges

def convert(mapping,spec,tupl):
    return (tupl[0], mapping[spec][tupl[1]],tupl[2],tupl[3])

class SymbolTable(object):
    """
    This class creates an instance of a symbol table.

    Args:
        static (bool): 

    Attributes:
        container (dict):
        static (bool): 
    """    
    def __init__(self, static = True):
        self.container = dict()
        self.static = static
    
    def insert(self, var_name, scope, value):
        """
        """
        if scope in self.container.keys():
            if var_name in self.container[scope].keys() and self.static:
                print "Variable {} is already defined".format(var_name)
                exit(1)
            else:
                if scope in ['ap','letter','system','spec']:
                    self.container[scope][var_name] = eval(value)
                elif scope == 'region':
                    self.container[scope][var_name] = value
                elif scope == 'aut':
                    if var_name == 'accepting' or var_name == 'initial':
                        self.container[scope][var_name] = map(int,value)
                    elif var_name == 'edges':
                        self.container[scope][var_name] = map(eval,value)
                    elif var_name == 'n_nodes':
                        self.container[scope][var_name] = eval(value)
                    else:
                        self.container[scope][var_name] = value
                elif value[0] == '[':
                    self.container[scope][var_name] = '('+value+')'
                elif value[0] != '(' or value[-1] != ')':
                    self.container[scope][var_name] = value
                else:
                    self.container[scope][var_name] = value  
        else:
            self.container[scope] = dict()
            if scope in ['ap','letter','system', 'spec']:
                self.container[scope][var_name] = eval(value)
            elif scope == 'region':
                self.container[scope][var_name] = eval(value)
            elif scope == 'aut':
                if var_name == 'accepting' or var_name == 'initial':
                    self.container[scope][var_name] = map(int,value)
                elif var_name == 'edges':
                    self.container[scope][var_name] = map(eval,value)
                elif var_name == 'n_nodes':
                    self.container[scope][var_name] = eval(value)
                else:
                    self.container[scope][var_name] = value
                
            else:
                self.container[scope][var_name] = value
    
    def lookup(self, var_name, scope, neg=False):
        """
        """
        try:
            if neg:
                value = str((-1*np.array(eval(self.container[scope][var_name]))).tolist())
                return value
            else:
                value = self.container[scope][var_name]
                return value
        except KeyError:
            print 'The scope {} or variable {} is undefined'.format(scope, 
                var_name)
            exit(1)
            
    def rename_states(self):
        """
        This routine goes over all the specifications and changes the 
        state names from frozensets to q_i.
        """
        mapping = {}
        keys = {}
        for spec, graph in self.container['spec'].iteritems():
            mapping[spec] = dict()
            keys[spec] = list(self.container['spec'][spec].graph.iteritems())
            i = 0
            for state, graph in keys[spec]:
                mapping[spec][state] = 'q_{}'.format(i)       
                self.container['spec'][spec].graph[mapping[spec][state]] = self.container['spec'][spec].graph[state]
                del self.container['spec'][spec].graph[state]
                i += 1
        for spec, graph in self.container['spec'].iteritems():
            i = 0
            for state, graph in keys[spec]: 
                length = len(list(self.container['spec'][spec].graph[mapping[spec][state]]))
                self.container['spec'][spec].graph[mapping[spec][state]] =\
                set(map(convert,[mapping]*length,[spec]*length,
                    list(self.container['spec'][spec].graph[mapping[spec][state]])))
                i += 1
        return self
                
    def get_tagged_nodes(symbol_table):
        """
        """
        if 'aut' not in symbol_table.container.keys():
            nodes = {}
            for spec in symbol_table.container['spec'].keys():
                nodes[spec] =  tag_nodes(symbol_table.container['spec'][spec].graph)
            for spec in symbol_table.container['spec'].keys():        
                for key in nodes[spec].keys():
                    nodes[spec][key]['region'] = nodes[spec][key]['region'].to_canon_tree()  
        else:
            new_keys = ['q_'+str(s) for s in range(symbol_table.container['aut']['n_nodes'])]
            nodes = dict.fromkeys(new_keys)
            for node in nodes.keys():
                nodes[node] = {}
                if isinstance(symbol_table.container['aut'][node],str):
                    nodes[node]['region'] = eval(symbol_table.container['aut'][node]).to_canon_tree()
                else:
                    nodes[node]['region'] = symbol_table.container['aut'][node].to_canon_tree()
                if int(node[2:]) in symbol_table.container['aut']['accepting']:
                    nodes[node]['accepting'] = True
                else:
                    nodes[node]['accepting'] = False
                    
                if int(node[2:]) in symbol_table.container['aut']['initial']:
                    nodes[node]['initial'] = True
                else:
                    nodes[node]['initial'] = False                    
                    
        return nodes

    def get_tagged_edges(symbol_table):
        """
        """
        if 'aut' not in symbol_table.container.keys():
            edges = {}
            for spec in symbol_table.container['spec'].keys():
                edges[spec] =  tag_edges(symbol_table.container['spec'][spec].graph)
            for spec in symbol_table.container['spec'].keys():        
                for key in edges[spec].keys():
                    edges[spec][key]['region'] = edges[spec][key]['region'].to_canon_tree() 
        else:
            new_keys = [s for s in symbol_table.container['aut'].keys() if isinstance(s,tuple)]
            edges = dict.fromkeys(new_keys)
            for key in new_keys:
                edges[key] = {}
                if isinstance(symbol_table.container['aut'][key],str):
                     edges[key]['region'] = eval(symbol_table.container['aut'][key]).to_canon_tree() 
                else:
                    edges[key]['region'] = symbol_table.container['aut'][key].to_canon_tree() 
        return edges
        



"""
.. module:: cvx_gen
   :platform: Unix
   :synopsis: 

.. moduleauthor:: Mohammed Alshiekh


"""

from region import *

def tree_leaf_count(tree):
    """
    This function takes a tree and returns its number of leaf.
    """
    n = 0
    if isinstance(tree,Workspace):
        return 0
    elif isinstance(tree,Terminal):
        return 1
    else:
        for operand in tree.operands:
            if isinstance(operand,Terminal):
                n += 1
            if isinstance(operand,Tree):
                n += tree_leaf_count(operand)
    return n

def total_leaf_count(nodes,edges):
    """
    This function takes a tagged automaton and returns the total number of leafs in all of its trees.
    """
    n = 0
    for key, node in nodes.iteritems():
        n += tree_leaf_count(node['region'])
    for key, edge in edges.iteritems():
        n += tree_leaf_count(edge['region'])
    return n

def _pre_cvx(nodes,edges,constants):
    string = '%dymanics\n'
    string += 'A = '+Matrix(constants['A']).mmat()+';\n'
    string += 'B = '+Matrix(constants['B']).mmat()+';\n'
    #string += 'n = size(A,1);\n'
    #string += 'm = size(B,2);\n'
    string += 'n = '+str(constants['n'])+';\n'
    string += 'm = '+str(constants['m'])+';\n'
    string += 'Q = '+Matrix(constants['Q']).mmat()+';\n'
    string += 'R = '+Matrix(constants['R']).mmat()+';\n'
    # the dimensions of the final state (origin) must be equal to the workspace's dimensions
    string += 'x0 = '+Matrix(constants['x0']).mmat()+';\n'
    # The initial position should be set by the user
    string += 'xf = '+Matrix(constants['xf']).mmat()+';\n'
    # state transition cost should be set by the user
    string += '%state transition cost\n'
    string += 's = 1;\n'
    string += '%number of states\n'
    string += 'Nq = {};\n'.format(len(nodes.keys()))         
    return string

def _cvx_prog(nodes,edges):
#[[1.0000e+00 0.0000e+00 ; 0.0000e+00 1.0000e+00 ], zeros(n,1), [1.0000e+00 ; 2.0000e+00 ] ; 0, 0, zeros(1,m); [1.0000e+00 ; 2.0000e+00 ]', zeros(1,1), [[4]]]
    def s_procedure(op):
        #[zeros(n), zeros(n,1), 0.5*[1.0000e+00 2.0000e+00 ]' ; zeros(1,n), 0, 0; 0.5*[1.0000e+00 2.0000e+00 ],0, -1*48]
        if isinstance(op,HalfSpace):
            if op.point.shape[0] == 1:
                return "\t\t- tau({i})*[zeros(n), zeros(n,1), 0.5*{c}' ; zeros(1,n), 0, 0; 0.5*{c}, 0, -1*{b}]...\n".format(i=c,b=op.b,c=Matrix(op.point).mmat())
            else:
                return "\t\t- tau({i})*[zeros(n), zeros(n,m), 0.5*{c} ; zeros(m,n), zeros(m), 0; 0.5*{c}', zeros(1,m), -1*{b}]...\n".format(i=c,b=op.b,c=Matrix(op.point).mmat())
        elif isinstance(op,Ellipsoid):
            return "\t\t- tau({i})*[{P}, zeros(n,1), {r} ; 0, 0, zeros(1,m); {r}', zeros(1,1), {c}]...\n".format(i=c,P=Matrix(op.A).mmat(),r=Matrix(op.b).mmat(),c=op.c)  
        elif isinstance(op,Workspace):
            return ""
        else:
            print 'This region is unknown: {}. The resulting cvx prog will probably be incorrect. Please modify the automaton.'.format(op)
            return ''
    
    c = 1     
    
    # starting the sdp program and declaring the variables
    string = 'cvx_begin sdp;\n'
    string += '\t variable P(n,n,Nq);\n'
    string += '\t variable r(n,1,Nq);\n'
    string += '\t variable t(1,Nq);\n'
    string += '\t variable tau({},1);\n'.format(total_leaf_count(nodes,edges))
    # maximizing the objective function
    string += "\t maximize( quad_form(x0,P(:,:,1)) + 2*r(:,:,1)'*x0 + t(:,1) );\n"
    # subject to:
    string += '\t for i=1:Nq\n'
    string += '\t\t P(:,:,i) == semidefinite(n);\n'
    string += '\t end\n'
    string += '\t tau(:) >= 0;\n'
    
    # state constraints
    for key, node in nodes.iteritems():
        string += '%{} \n'.format(key)
        tree = node['region']

        if isinstance(tree,Terminal):
            tree = tree.to_tree()

        for operand in tree.operands:
            string += '\n'
            if isinstance(operand,Tree):
                string += "\t 0 <= [A'*P(:,:,{i}) + P(:,:,{i})'*A + Q, P(:,:,{i})*B,  A'*r(:,:,{i}); ...\n".format(i=int(key[2:])+1)
                string += "\t       B'*P(:,:,{i})',                  R,           B'*r(:,:,{i}); ...\n".format(i=int(key[2:])+1)
                string += "\t       r(:,:,{i})'*A,                   r(:,:,{i})'*B, 0  ]...\n".format(i=int(key[2:])+1)
                for op in operand.operands:
                    if not isinstance(op,Workspace):
                        string += s_procedure(op)
                        if len(string) > 1:
                            c += 1
                string = string[:-4]
                string += ';\n'       
# [P(:,:,2),    r(:,:,2),   zeros(n,1); ...
#            r(:,:,2)',   0,   0; ...
#            zeros(1,n),  0, t(:,2) + s]               
	# transition constraints
    for key, edge in edges.iteritems():
        string += '\n'
        tree = edge['region']

        if isinstance(tree,Terminal):
            tree = tree.to_tree()

        for operand in tree.operands:
            string += '%{} \n'.format(key)
            if isinstance(operand,Tree):
                string += "\t 0 <= [P(:,:,{i}),    r(:,:,{i}),   zeros(n,1); ...\n".format(i=int(key[1][2:])+1)
                string += "\t       r(:,:,{i})',            0,            0; ...\n".format(i=int(key[1][2:])+1)
                string += "\t       zeros(1,n),             0, t(:,{i}) + s] ...\n".format(i=int(key[1][2:])+1)
                string += "\t     -[P(:,:,{i}),    r(:,:,{i}),   zeros(n,1); ...\n".format(i=int(key[0][2:])+1)
                string += "\t       r(:,:,{i})',            0,            0; ...\n".format(i=int(key[0][2:])+1)
                string += "\t       zeros(1,n),    0, t(:,{i}) + s] ...\n".format(i=int(key[0][2:])+1)
                for op in operand.operands:
                    if not isinstance(op,Workspace):
                        string += s_procedure(op)
                        if len(string) > 1:
                            c += 1
                string = string[:-4]
                string += ';\n'
                
    # accepting state contraints        
    for key, node in nodes.iteritems():
        if node['accepting']:
            string += '% final \n'  
            string += "\tquad_form(xf,P(:,:,{i})) + 2*r(:,:,{i})'*xf + t(:,{i}) == 0;\n".format(i=int(key[2:])+1)
    string += 'cvx_end;'
    return string
      
def to_cvx(nodes,edges,constants):
    """
    This function assumes that all trees are in canonical form.
    """
    string = _pre_cvx(nodes,edges,constants)
    string += _cvx_prog(nodes,edges)
    return string


"""
.. module:: parser
   :platform: Unix
   :synopsis: 

.. moduleauthor:: Mohammed Alshiekh


"""

from pyparsing import *
from symbol_table import SymbolTable
import sys
from region import *
from Automata import RegularLanguage

def printf(text):
   print text

def tup2trans(string):
    tup = eval(string)
    new_tup = ('q_{}'.format(tup[0]),'q_{}'.format(tup[1]))
    return new_tup

def parse_miu(file_):
    parser = Parser()
    aut_begin = 0
    constants_begin = 0
    system_begin = 0
    specs_begin = 0
    ap_begin = 0
    let_begin = 0
    # getting blocks line numbers
    f = open(file_, 'r').readlines()
    for i,line in enumerate(f):
        # if 'dimensions' in line[:len('dimensions')] and not 'end' in line :
        #     dim_begin = i+1
        # if 'dimensions' in line and 'end' in line[:len('end')]:
        #     dim_end = i
        if 'constants' in line[:len('constants')] and not 'end' in line:
            constants_begin = i+1
        if 'constants' in line and 'end' in line[:len('end')]:
            constants_end = i
        if 'regions' in line[:len('regions')] and not 'end' in line:
            region_begin = i+1
        if 'regions' in line and 'end' in line[:len('end')]:
            region_end = i
        if 'system' in line[:len('system')] and not 'end' in line:
            system_begin = i+1
        if 'system' in line and 'end' in line[:len('end')]:
            system_end = i
        if 'automaton' in line[:len('automaton')] and not 'end' in line:
            aut_begin = i+1
        if 'automaton' in line and 'end' in line[:len('end')]:
            aut_end = i
        if 'specifications' in line[:len('specifications')] and not 'end' in line:
            specs_begin = i+1
        if 'specifications' in line and 'end' in line[:len('end')]:
            specs_end = i
        if 'ap' in line[:len('ap')] and not 'end' in line:
            ap_begin = i+1
        if 'ap' in line and 'end' in line[:len('end')]:
            ap_end = i
        if 'letters' in line[:len('letters')] and not 'end' in line:
            let_begin = i+1
        if 'letters' in line and 'end' in line[:len('end')]:
            let_end = i
    
    if constants_begin > 0:
        parser.const_rule.parseString(''.join(f[constants_begin:constants_end]))    
    
    if system_begin > 0:
        parser.system_rule.parseString(''.join(f[system_begin:system_end]))
    
    if region_begin > 0:
        parser.region_rule.parseString(''.join(f[region_begin:region_end]))
    
    if ap_begin > 0:
        parser.ap_rule.parseString(''.join(f[ap_begin:ap_end]))
    
    if let_begin > 0:
        parser.let_rule.parseString(''.join(f[let_begin:let_end]))
    
    if specs_begin > 0:
        parser.spec_rule.parseString(''.join(f[specs_begin:specs_end]))
    
    if aut_begin > 0:
        parser.aut_rule.parseString(''.join(f[aut_begin:aut_end]))
        return parser.symbol_table
    else:
        return parser.symbol_table.rename_states() 

class Parser(object):
    """
    This class creates an instance that is used to parse a \*.miu file.

    Args:
        None

    Attributes:
        | symbol_table (SymbolTable):
        | const_rule: The rules for parsing the Constant block.
        | let_rule: The rules for parsing the Letter block.
        | spec_rule: The rules for parsing the Spec block.
        | region_rule: The rules for parsing the Region block.
        | ap_rule: The rules for parsing the AP block.
        | system_rule: The rules for parsing the System block.
    """
    def __init__(self):

        self.symbol_table = SymbolTable()     
        comment = (Literal('#') + restOfLine).suppress()
        point = Literal( "." )
        num = Combine(Optional("-") + Word( nums, nums ) + 
                           Optional( point + Optional( Word( nums ) ) ))
        semi = Literal(';').suppress()
        equals = Literal('=').suppress()
        bind = Literal('<-').suppress()
        negate = Literal('-')

        variable_lhs = Word(alphas, alphanums + '_')
        variable_rhs = Word(alphas, alphanums + '_').setParseAction(lambda s, loc, toks: self.symbol_table.lookup(toks[0],'constant'))
        negated_variable_rhs = (negate+Word(alphas, alphanums + '_')).setParseAction(lambda s, loc, toks: self.symbol_table.lookup(toks[1],'constant',neg=True))
        expr = Forward()
        matrix = Literal('[') + expr + ZeroOrMore(',' + expr) + Literal(']')
        listoflists = Literal('[') + matrix + ZeroOrMore(',' + matrix) + Literal(']') #Optional('[') + matrix + ZeroOrMore(',' + matrix) + Optional(']')
        operand = num| variable_rhs | matrix | negated_variable_rhs
        factor = operand | expr
        term = factor + ZeroOrMore( oneOf('* /') + factor )
        expr << Optional('(') + term + ZeroOrMore(oneOf('+ -') + term) + Optional(')') 

        # constant block assignment expression
        const_assigmentExp = (variable_lhs + equals + Combine(expr) + semi).setParseAction(lambda s, loc, toks:\
                                                                                           self.symbol_table.insert(toks[0],'constant',toks[1]))

        # Region block assignment expression
        region_expr = Forward()
        region_variable_rhs = Word(alphas, alphanums + '_').setParseAction(lambda s, loc, toks: self.symbol_table.lookup(toks[0],'region'))
        region_term = MatchFirst([(Optional('(') + ((Literal('HalfSpace(') | Literal('Ellipsoid(')) + Combine(expr)\
                       + Literal(',') + Combine(expr) + Optional(')')  + Optional(')')) | Literal('Empty()') | Literal('Workspace()') ) ,\
                                  (Optional('(') + region_variable_rhs + Optional(')'))])
        region_expr << Optional('(') + region_term + ZeroOrMore(oneOf('& |') + region_term) + Optional(')')
        region_assigmentExp = (variable_lhs + equals + Combine(region_expr) + semi).setParseAction(lambda s, loc, toks: \
                                                                                                   self.symbol_table.insert(toks[0],'region',toks[1]))

        # System block assignment expression
        system_assigmentExp = (variable_lhs + equals + Combine(expr) + semi).setParseAction(lambda s, loc, toks:\
                                                                                            self.symbol_table.insert(toks[0],'system',toks[1]))

        # AP block assignment expression
        ap_expr = Optional('(') + region_term + ZeroOrMore(oneOf('& |') + region_term) + Optional(')')
        ap_assigmentExp = (variable_lhs + bind + Combine(ap_expr) + semi).setParseAction(lambda s, loc, toks:\
                                                                                         self.symbol_table.insert(toks[0],'ap',toks[1]))

        # Spec block assignment expression
        spec_term = MatchFirst([Word(alphas,'*'),Word(alphas,exact=1)]).setParseAction(lambda s, loc, toks:self.check_letter(toks))
        spec_expr = (Optional('(') + Literal('RE(').setParseAction(lambda s, loc, toks:\
                      'RegularLanguage("') + ZeroOrMore(spec_term)+ Word(')').setParseAction(lambda s,\
                       loc, toks: '",self.container).recognizer.asDFA().to_graph(self.container)')  + Optional(')'))
        spec_assigmentExp = (variable_lhs + equals + Combine(spec_expr) + semi).setParseAction(lambda s, loc, toks:\
                                                                                               self.symbol_table.insert(toks[0],'spec',toks[1]))

        # Letter block assignment expression
        let_expr = MatchFirst([Combine(Word('{')+Word('}')).setParseAction(lambda s, loc, toks:"set()".format(toks[0])),Word('{') + Word(alphas, exact=1).setParseAction(lambda s, loc, toks:"'{}'".format(toks[0]))\
                   + ZeroOrMore(',' + Word(alphas).setParseAction(lambda s, loc, toks:"'{}'".format(toks[0]))) + Word('}')])

        let_assigmentExp = (variable_lhs + equals + Combine(let_expr) + semi).setParseAction(lambda s, loc, toks:\
                                                                                             self.symbol_table.insert(toks[0],'letter',toks[1]))

        # Automaton block assignment expression
        comma = Literal(',').suppress()
        left_bracket = Literal('[').suppress()
        right_bracket = Literal(']').suppress()
        region_rhs = Word(alphanums).setParseAction(lambda s, loc, toks: self.symbol_table.lookup(toks[0],'region'))
        #
        
        index = left_bracket+Word(nums)+right_bracket
        tup = Combine(Literal('(')+Word(nums)+Literal(',')+Word(nums)+Literal(')'))
        t_index = left_bracket+tup+right_bracket
        edges_list = left_bracket + ZeroOrMore(tup+comma) + tup + right_bracket
        alist = left_bracket + ZeroOrMore(Word(nums)+comma) + Word(nums) + right_bracket
        n_nodes_aut = Literal('n_nodes') + equals + Word(nums)
        edges_aut = Literal('edges') + equals + edges_list
        accepting_aut = Literal('accepting') + equals + alist
        initial_aut = Literal('initial') + equals + alist
        node_bind_aut = Literal('nodes')+ index + bind + region_rhs
        edge_bind_aut = Literal('edges')+ t_index + bind + region_rhs
        aut_variable_lhs = Literal('edges') |\
                           Literal('n_nodes') |\
                           Literal('accepting') |\
                           Literal('initial') |\
                           Literal('nodes[') + Literal(nums) + Literal(']')
        
        aut_assignmentExp = (n_nodes_aut.setParseAction(lambda s, loc, toks:\
                                                   self.symbol_table.insert(toks[0],'aut',toks[1])) |\
                            edges_aut.setParseAction(lambda s, loc, toks:\
                                                   self.symbol_table.insert(toks[0],'aut',toks[1:])) |\
                            accepting_aut.setParseAction(lambda s, loc, toks:\
                                                   self.symbol_table.insert(toks[0],'aut',toks[1:])) |\
                            initial_aut.setParseAction(lambda s, loc, toks:\
                                                   self.symbol_table.insert(toks[0],'aut',toks[1:])) |\
                            node_bind_aut.setParseAction(lambda s, loc, toks:\
                                                   self.symbol_table.insert('q_'+toks[1],'aut',toks[2])) |\
                            edge_bind_aut.setParseAction(lambda s, loc, toks:\
                                                   self.symbol_table.insert(tup2trans(toks[1]),'aut',toks[2])))\
                            + semi
                  
        const_parse = const_assigmentExp     
        system_parse = system_assigmentExp
        region_parse = region_assigmentExp
        ap_parse = ap_assigmentExp
        let_parse = let_assigmentExp
        spec_parse = spec_assigmentExp
        aut_parse = aut_assignmentExp

        self.const_rule = OneOrMore(Group(const_parse)) 
        self.const_rule.ignore(comment) 
        self.let_rule = OneOrMore(Group(let_parse))        
        self.let_rule.ignore(comment)
        self.spec_rule = OneOrMore(Group(spec_parse))         
        self.spec_rule.ignore(comment)
        self.ap_rule = OneOrMore(Group(ap_parse))        
        self.ap_rule.ignore(comment)
        self.region_rule = OneOrMore(Group(region_parse))        
        self.region_rule.ignore(comment)
        self.system_rule = OneOrMore(Group(system_parse))        
        self.system_rule.ignore(comment) 
        self.aut_rule = OneOrMore(Group(aut_parse))        
        self.aut_rule.ignore(comment) 
        
    def check_letter(self, letter):
        """
        This method will inform the user if the letter does not exist in symbol_table.container['letter'] keys, then quits.

        Args:
            letter (str): The letter to be checked if exist as a key.

        Returns:
            None
        """
        if letter[0][0] not in self.symbol_table.container['letter'].keys():
            print 'Letter {} is not defined'.format(letter[0][0])
            sys.exit('check_letter')




"""Util.py

Simple utility functions for PADS library.
D. Eppstein, April 2004.
"""

def arbitrary_item(S):
    """
    Select an arbitrary item from set or sequence S.
    Avoids bugs caused by directly calling iter(S).next() and
    mysteriously terminating loops in callers' code when S is empty.
    """
    try:
        return next(iter(S))
    except StopIteration:
        raise IndexError("No items to select.")

def map_to_constant(constant):
    """
    Return a factory that turns sequences into dictionaries, where the
    dictionary maps each item in the sequence into the given constant.
    Appropriate as the adjacency_list_type argument for Graphs.copyGraph.
    """
    def factory(seq):
        return dict.fromkeys(seq,constant)
    return factory


from collections import defaultdict
import inspect
class Graph(object):
    """ Graph data structure, directed by default. """

    def __init__(self, connections = None, directed=True):
        self.graph = defaultdict(set)
        self._directed = directed
        # if connections is not None:
        #     self.add_connections(connections)

    # def add_connections(self, connections):
    #     """ Add connections (list of tuple pairs) to graph """

    #     for node1, node2 in connections:
    #         self.add(node1, node2)

    def add(self, node1, letter, node2, accepting=False,initial=False):
        """ Add connection between node1 and node2 """
        # curframe = inspect.currentframe()
        # calframe = inspect.getouterframes(curframe, 2)
        # print 'caller name:', calframe[1][3]
        self.graph[node1].add((letter,node2,accepting,initial))
        #self.graph[node1].add((1,node2,terminal))

    def remove(self, node):
        """ Remove all references to node """

        for n, cxns in self.graph.iteritems():
            try:
                cxns.remove(node)
            except KeyError:
                pass
        try:
            del self._graph[node]
        except KeyError:
            pass

    # def is_connected(self, node1, node2):
    #     """ Is node1 directly connected to node2 """

    #     return node1 in self.graph and node2 in self.graph[node1]
    
    # def __str__(self):
    #     return '{}({})'.format(self.__class__.__name__, dict(self.graph))
    
    # def __repr__(self):
    #     return '{}({})'.format(self.__class__.__name__, dict(self.graph))


from setuptools import setup

def readme():
    with open('README.rst') as f:
        return f.read()

def version():
    with open('sydar/version.txt') as f:
        v= f.read()
        return v

setup(name='sydar',
      version=version(),
      description='Synthesis Done Approximately Right',
      long_description=readme(),
      url='https://github.com/u-t-autonomous/sydar.git',
      author='Mohammed Alshiekh',
      author_email='sahabi@gmail.com',
      license='BSD',
      packages=['sydar'],
      install_requires=[
          'scipy',
          'pyparsing',
          'numpy',
          ],
      entry_points = {
        'console_scripts': ['sydar-matlab=sydar.command_line:main'],
        },
      test_suite='nose.collector',
      tests_require=['nose'],
      zip_safe=False)


"""
.. module:: region
   :platform: Unix
   :synopsis: 

.. moduleauthor:: Mohammed Alshiekh

This module contains many functions that are developed in the ast 
module for TuLiP and can be found here:
https://github.com/tulip-control/tulip-control/blob/master/tulip/spec/ast.py


"""

from abc import ABCMeta, abstractmethod
from utils import *
from copy import deepcopy
from random import random

OPMAP = {'||': '||', '&': '&','!': '!'}

def hierarchy_pos(G, root, width=1., vert_gap = 0.2, vert_loc = 0, xcenter = 0.5 ):
    '''If there is a cycle that is reachable from root, then result 
       will not be a hierarchy.

       G: the graph
       root: the root node of current branch
       width: horizontal space allocated for this branch - 
       avoids overlap with other branches
       vert_gap: gap between levels of hierarchy
       vert_loc: vertical location of root
       xcenter: horizontal location of root
    '''
    neighbors = G.neighbors(root)
    def h_recur(G, root, width=1., vert_gap = 0.2, vert_loc = 0, xcenter = 0.5,
     poss = None, parent = None, parsed = [] ):
        if(root not in parsed):
            parsed.append(root)
            if poss == None:
                poss = {root:(xcenter,vert_loc)}
            else:
                poss[root] = (xcenter, vert_loc)
            neighbors = G.neighbors(root)
            if len(neighbors)!=0:
                dx = width/len(neighbors) 
                nextx = xcenter - width/2 - dx/2
                for neighbor in neighbors:
                    nextx += dx
                    poss = h_recur(G,neighbor, width = dx, vert_gap = vert_gap,
                            vert_loc = vert_loc-vert_gap, xcenter=nextx, 
                            poss=poss, parent = root, parsed = parsed)
        return poss

    return h_recur(G, root, width=1., vert_gap = 0.2, 
        vert_loc = 0, xcenter = 0.5)

class RegionNode(object):
    """Base class for all nodes."""

    __metaclass__ = ABCMeta
    opmap = OPMAP

    @abstractmethod
    def __init__(self):
        pass

    #@abstractmethod
    #def __repr__(self):
    #    pass

    @abstractmethod
    def flatten(self):
        pass

    def __and__(self, other):
        return Tree('&', [self, other])

    def __or__(self, other):
        return Tree('||', [self, other])

    def complement(self):
        return Tree('!', self)

    def over_approx(self):
        return RegionNode()

    def over_approx(self):
        return RegionNode()  
    
    def __eq__(self, other):
        return self.flatten() == other.flatten()
    
    def empty_clear_complete(self):
        return True      

    def clear_single_complete(self):
        """
        Checks if a tree has any single leafed nodes. 
        If yes, it returns False, else True.
        """
        return True

class Terminal(RegionNode):
    """Base class for all terminal nodes."""
    def __init__(self, approx=False):
        self.approx = approx

    def insert_intersection(self):
        return Tree('&', [self])        

    def redund_clear_complete(self):
        """
        Checks if a tree has any redundant nodes. 
        If yes, it returns False, else True.
        """
        return True

    def to_canon_tree(self):
        node = deepcopy(self)
        tree = Tree('||',[Tree('&',[node], level = 2)], level = 1)
        return tree
    
class Tree(RegionNode):
    def __init__(self, operator, operands, level = None):
        try:
            operator + 'a'
        except TypeError:
            raise TypeError(
                'operator must be string, got: {op}'.format(
                    op=operator))
        self.type = 'operator'
        self.operator = operator
        self.operands = list(operands)
        self.level = level

    @property
    def operator(self):
        """I'm the '_operator' property."""
        return self._operator

    @operator.setter
    def operator(self, value):
        if value in self.opmap:
            self._operator = value
        else:
            print "The operator is not part of the OPMAP: {}".format(
                " ".join(self.opmap))
            sys.exit(1)

    @operator.deleter
    def operator(self):
        del self._operator          

    def __repr__(self):
        return self.flatten()
        return '{t}({op}, {xyz})'.format(
            t=type(self).__name__,
            op=repr(self.operator),
            xyz=', '.join(repr(x) for x in self.operands))

    def __str__(self, depth=None):
        return self.flatten_infix()
        if depth is not None:
            depth = depth - 1
        if depth == 0:
            return '...'
        return '({op},{xyz})'.format(
            op=self.operator,
            xyz=','.join(x.__str__(depth=depth)
                         for x in self.operands))

    def __len__(self):
        return 1 + sum(len(x) for x in self.operands)

    def flatten(self):
        return ' '.join([
            '(',
            self.opmap[self.operator],
            ', '.join(x.flatten() for x in self.operands),
            ')'])  

    def flatten_infix(self):
        return ' '.join([
            '(',
            '{} '.join(x.flatten() for x in self.operands).format(self.opmap[self.operator]),
            ')'])  
    
    def insert_intersection(self):
        if self.operator == '&':
            self.level = 2
            return self
        else:
            return Tree('&', [self], level = 2)
    
    def show(self):
        import networkx as nx
        import matplotlib.pyplot as plt
        self.colors = []
        self.node_counter = 2
        self.G = nx.Graph()
        self.labels = {}
        self.G.add_node(1)
        self.colors.append('g')
        self.labels[1] = self.operator
        self.parse(1)
        poss = hierarchy_pos(self.G, root=1)  
        nx.draw(self.G, pos=poss, with_labels=False)
        nx.draw_networkx_nodes(self.G, pos=poss, node_color=self.colors)
        self.G = nx.relabel_nodes(self.G, self.labels)
        nx.draw_networkx_labels(self.G, pos=poss, labels=self.labels) 
        plt.show()
        
    def parse(self, parent, region=None):
        if region is None:
            region = self
        for operand in region.operands:
            if isinstance(operand, Terminal):
                # Add node
                self.G.add_node(self.node_counter)
                self.colors.append('r' if operand.approx else 'g')
                self.G.add_edge(parent, self.node_counter)
                #print("Added edge from", parent, "to", self.node_counter)
                self.labels[self.node_counter] = type(operand).__name__
                self.node_counter += 1
            elif isinstance(operand, RegionNode):
                # Add sub-tree
                self.G.add_node(self.node_counter)
                self.colors.append('g')
                self.G.add_edge(parent, self.node_counter)
                self.labels[self.node_counter] = operand.operator
                self.node_counter += 1
                self.parse(self.node_counter-1, operand)
            else:
                raise ValueError('Incorrect operand found by parse()')

    def to_canon_tree(self):
        """
        This routine transforms any tree to a canonical tree.
        """
        tree = deepcopy(self)
        done = False
        while not done:
            tree = tree.extend_tree()
            tree = tree.lossless_transform()
            done = True
        #tree = tree.lossy_transform()
        return tree

    def extend_tree(self):
        tree = deepcopy(self)
        if tree.operator != '||':
            tree = Tree('||',[tree], level = 1)
        new_operands = []
        for operand in tree.operands:
            if operand is not None:
                new_operands.append(operand.insert_intersection())
        tree.operands = new_operands
        return tree

    def lossless_transform(self):
        """
        This routine applies lossless transformations to the tree.
        """
        tree = deepcopy(self)
        tree = tree.fix_useless_nodes()
        tree = tree.fix_empty_region()
        return tree

    def lossy_transform(self):
        """
        This routine transforms any prepared tree to a canonical tree.
        """
        tree = deepcopy(self)
        for i, a_tree in enumerate(tree.operands):
            complete = a_tree.all_terminal()
            while not all(complete):
                if len(np.array(a_tree.operands)[complete].tolist()) == 0:
                    new_operands = []
                else:
                     new_operands = np.array(a_tree.operands)[complete].tolist()
                for operand in np.array(a_tree.operands)[~complete]:
                    new_operands.append(operand.to_terminal())
                tree.operands[i].operands = new_operands
                complete = a_tree.all_terminal()
        return tree
        
    def fix_useless_nodes(self):
        """
        This routine removes any redundant node from any prepared tree.
        """
        tree = deepcopy(self)
        if tree.operator == '||':
            for i, operand in enumerate(tree.operands):
                if operand.operator == '&':
                    tree.operands[i] = operand.clear_useless()
                else:
                    print "The tree is not prepared (2nd level)"
        else:
            print "The tree is not prepared (1st level)"
        return tree

    def clear_useless(self):
        """
        Transforms any tree by removing all redundant nodes
        """
        complete = self.useless_clear_complete() and self.redund_clear_complete()
        while not complete:
            for i, operand in enumerate(self.operands):
                if isinstance(operand, Tree):
                    if len(operand.operands) < 2:
                        self.operands.remove(operand)
                        self.operands += operand.operands
                    elif operand.operator == self.operator:
                        self.operands.remove(operand)
                        self.operands += operand.operands
                    else:
                        self.operands[i] = operand.clear_useless()
            complete = self.useless_clear_complete() and self.redund_clear_complete()
        return self    

    def useless_clear_complete(self):
        """
        Checks if a tree has any useless nodes. 
        If yes, it returns False, else True.
        """
        flag = True
        for operand in self.operands:
            if isinstance(operand, Tree):
                if len(operand.operands) < 2:
                    return False
                else:
                    flag = operand.useless_clear_complete()
        return flag    

    def redund_clear_complete(self):
        """
        Checks if a tree has any redundant nodes. 
        If yes, it returns False, else True.
        """
        flag = True
        for operand in self.operands:
            if isinstance(operand, Tree):
                if self.operator == operand.operator:
                    return False
                else:
                    flag = operand.redund_clear_complete()
        return flag

    def fix_empty_region(self):
        tree = deepcopy(self)
        if tree.operator == '||':
            for i, operand in enumerate(tree.operands):
                if operand.operator == '&':
                    tree.operands[i] = operand.clear_empty()
                else:
                    print "The tree is not prepared (2nd level)"
        else:
            print "The tree is not prepared (1st level)" 
        return tree

    def clear_empty(self):
        tree = deepcopy(self)
        complete = tree.empty_clear_complete()
        while not complete:
            for i, subtree in enumerate(tree.operands):
                if isinstance(subtree, Empty):
                    if tree.operator == '||':
                        if all(map(isinstance,tree.operands,
                            [Empty]*len(tree.operands))):
                            tree = Empty()
                        else:    
                            tree.operands.remove(subtree)
                    elif tree.operator == '&' and tree.level != 2:
                        tree = Empty()
                        break
                    elif tree.operator == '&' and tree.level == 2:
                        tree.operands = [Empty()]
                elif isinstance(subtree, Tree):
                    tree.operands[i] = subtree.clear_empty()
            complete = tree.empty_clear_complete()
        return tree
  
    def empty_clear_complete(self):
        flag = True
        for operand in self.operands:
            if isinstance(operand, Empty):
                if self.level == 2 and len(self.operands) == 1:
                    pass
                else:
                    return False
            elif isinstance(operand, Tree):
                flag = operand.empty_clear_complete()
        return flag            

    def all_terminal(self):
        terminal = np.array(map(isinstance,self.operands,
            [Terminal]*len(self.operands)))
        return terminal    

    def to_terminal(self):
        tree = deepcopy(self)
        terminal = tree.all_terminal()
        while not all(terminal):
            for i,operand in enumerate(self.operands):
                tree.operands[i] = operand.to_terminal()
            terminal = tree.all_terminal()
        return tree.over_approx()  

    def over_approx(self):
        tree = deepcopy(self)
        if tree.operator == '||':
            return self.over_approx_or()
        elif tree.operator == '&':
            return self.over_approx_and()
        else:
            print "unidentified operator {}".format(tree.operator)    

    def over_approx_or(self):
        try:
            from cvxpy import (Variable, Semidef, Variable, Minimize, Problem, 
            log_det, bmat)
            tree = deepcopy(self)
            tau = Variable(len(tree.operands))
            X = Semidef(tree.operands[0].P.shape[0])
            b_ = Variable(tree.operands[0].P.shape[0])
            constraints = []
            for i in range(len(tree.operands)):   
                X11 = X - tau[i]*tree.operands[i].A
                X12 = b_ - tau[i]*tree.operands[i].b
                X13 = np.zeros((tree.operands[0].P.shape[0],tree.operands[0].P.shape[0]))
                X21 = (b_ - tau[i]*tree.operands[i].b).T
                X22 = -1 - tau[i]*tree.operands[i].c
                X23 = b_.T
                X31 = np.zeros((tree.operands[0].P.shape[0],tree.operands[0].P.shape[0]))
                X32 = b_
                X33 = -X
                constraints.append((bmat([[X11, X12, X13], [X21, X22, X23],
                 [X31, X32, X33]]) << 0))
                constraints.append(tau[i] >= 0)    
            objective = Minimize(-log_det(X))
            prob = Problem(objective, constraints)
            result = prob.solve(solver='CVXOPT')
            A = np.array(X.value)
            b = np.array(b_.value)
            c = np.dot(b.T,b) - 1
            return Ellipsoid(A=A,b=b,c=c,approx=True)
        except:
            "Make sure it's an ellipse!"
            return Empty()

    def over_approx_and(self):
        return Empty(approx=True)            
            
class CanonTree(Tree):
    def __init__(self, tree, approx=False):
        self.operator = tree.operator
        self.operands = tree.operands
        self.approx = approx    
    
class HalfSpace(Terminal):
    def __init__(self, point, vector, approx=False):
        Terminal.__init__(self, approx)
        self.point = np.array(point)
        self.shape = self.point.shape
        self.vector = np.transpose(np.array(vector))
        self.b = np.dot(self.vector, np.transpose(self.point))[0][0]

    @property
    def point(self):
        """I'm the 'coord' property."""
        return self._point

    @point.setter
    def point(self, value):
        if is_point(value):
            self._point = np.transpose(value)
        elif is_point(np.transpose(value)):
            self._point = value
        else:
            print value
            print "The input must be a point, the given is {}".format(value)
            sys.exit(1)

    @point.deleter
    def point(self):
        del self._point  

    @property
    def vector(self):
        """I'm the 'coord' property."""
        return self._vector

    @vector.setter
    def vector(self, value):
        if is_point(value):
            if value.shape[0] == self.point.shape[0] and value.shape[1] == self.point.shape[1]:
                self._vector = value
            elif value.shape[0] == self.point.shape[1] and value.shape[1] == self.point.shape[0] and value.shape[0]==1:
                self._vector = value
            else:
                print "The vector's dimensions {} doesn't agree with the"\
                " point's dimensions {}".format(value.shape, self.point.shape)
                sys.exit(1)
        else:
            print "The input must be a vector, the given is {}".format(value)
            sys.exit(1)

    @vector.deleter
    def vector(self):
        del self._vector        
        
    def __str__(self, depth=None):
        return 'HalfSpace ({}, {})'.format(self.point.tolist(), 
            self.vector.tolist()) 
    
    def __repr__(self):
        return 'HalfSpace ({}, {})'.format(self.point, self.vector)          

    def flatten(self):
        return 'HalfSpace ({}, {})'.format(self.point, self.vector)     
    
class Empty(Terminal):
    def __init__(self, approx=False):
        Terminal.__init__(self, approx)
        self.name = 'Empty'

    def __str__(self, depth=None):
        return "Empty()"                 

    def __repr__(self):
        return "{}".format(self.name)  

    def flatten(self):
        return "{}".format(self.name)  

class Workspace(Terminal):
    def __init__(self, approx=False):
        Terminal.__init__(self, approx)
        self.name = 'Workspace'

    def __str__(self, depth=None):
        return "Workspace()"                 

    def __repr__(self):
        return "{}".format(self.name)  

    def flatten(self):
        return "{}()".format(self.name)  
       
class Ellipsoid(Terminal):
    def __init__(self, P=None, xc=None, A=None, b=None, c=None, approx=False):
        Terminal.__init__(self, approx)
        if A is None or b is None or c is None:
            self.P = PDMatrix(P)
            self.xc = CenterPoint(xc)
            self.A = P
            self.b = np.dot(np.array(P),np.array(xc))
            self.c = np.dot(np.dot(np.array(xc).transpose(),P),np.array(xc)) - 1
        else:
            self.P = P
            self.xc = xc
            self.A = A
            self.b = b
            self.c = c
      
    @property
    def P(self):
        """
        I'm the 'P' property.
        """
        return self._P
    
    @P.setter
    def P(self, value):
        if is_PD(value) or value==None:
            self._P = value
        else:
            print "The matrix P is not a PD matrix!"
            sys.exit(1)

    @P.deleter
    def P(self):
        del self._P   
        
    @property
    def xc(self):
        """
        I'm the 'c' property.
        """
        return self._xc
    
    @xc.setter
    def xc(self, value):
        if is_center_point(value):
            if dim_check(self.P, value):
                self._xc = value
            else:
                print "The dimensions are not compatible. P is {} while xc is"\
                " {}. ".format(self.P.shape,value.shape[0])
                sys.exit(1)
        elif value==None:
            self._xc = value
        else:
            print "The point xc is not a center point!"
            sys.exit(1)

    @xc.deleter
    def xc(self):
        del self._xc          

    def __str__(self, depth=None):
        return 'Ellipsoid ({}, {})'.format(self.P.matrix.tolist(), 
            self.xc.coord.tolist())     

    def __repr__(self):
        if self.P is not None:
            return 'Ellipsoid ({}, {})'.format(self.P.matrix, self.xc.coord)
        else:
            return 'Ellipsoid (A:{}, b:{}, c:{})'.format(self.A, self.b, self.c)

    def flatten(self):
        return 'Ellipsoid ({}, {})'.format(self.P.matrix, self.xc.coord)

    def plot(self,color='r'):
        import matplotlib.pyplot as plt
        n = 100000
        lis = np.zeros((n,2))
        for i in range(0,n,4):
            pos1 = random() * 5
            neg1 = random() * -5
            pos2 = random() * 5
            neg2 = random() * -5
            lis[i] = np.array([pos1,pos2])
            lis[i+1] = np.array([neg1,pos2])
            lis[i+2] = np.array([pos1,neg1])
            lis[i+3] = np.array([neg1,neg2])
        y1 = np.array([x for x in lis if (-.1 <= np.dot(np.dot(x.T,self.A),x) 
            + 2*np.dot(self.b.T,x) + self.c <= 0)])
        plt.scatter(y1[:,1],y1[:,0],color=color)
        
    
    def show(self):
        import matplotlib.pyplot as plt
        plt.show()
        
# def plot_multiple(ellipse_list):
#     for ellipse in ellipse_list:
#         ellipse.plot()
#     ellipse_list[0].show()
        
# def main():
#     c = [1,2]
#     A = [[1,0],[0,1]]
#     E = Ellipsoid(A,c)
#     h = HalfSpace([1,2],[4,5])
#     th = Empty()

# if __name__ == '__main__':
#     main()


"""Automata.py

Manipulation of and conversions between regular expressions,
deterministic finite automata, and nondeterministic finite automata.

D. Eppstein, UC Irvine, November 2003.
"""

from Util import arbitrary_item

import sys
import operator
import unittest

from PartitionRefinement import PartitionRefinement
from Sequence import Sequence
from Graph import Graph
# Hack for Python 3 compatibility
try:
    unicode
except:
    unicode = str

class LanguageError(Exception): pass

def Language(A):
    """Convert automaton A into an object describing its language.
    This is distinct from class RegularLanguage in case we
    want to later add other types of automaton and nonregular languages.
    """
    return A.language()

class RegularLanguage:
    """Object representing the language recognized by a DFA or NFA.
    Available operations are testing whether a string is in the language,
    logical combinations, and subset and equality testing.
    """
    def __init__(self,arg,symbol_table=None):
        if isinstance(arg,FiniteAutomaton):
            self.recognizer = arg
        elif isinstance(arg,(str,unicode)):
            self.symbol_table = symbol_table
            self.recognizer = RegExp(arg, self.symbol_table)
            self.graph = self.recognizer.asDFA().to_graph(symbol_table)
        else:
            raise LanguageError("Unrecognized constructor for RegularLanguage")

    def __contains__(self,inputsequence):
        return self.recognizer(inputsequence)

    def __eq__(self,other):
        if not isinstance(other,RegularLanguage):
            return None
        return self.recognizer.minimize() == other.recognizer.minimize()

    def __ne__(self,other):
        return not (self == other)

    def __le__(self,other):
        return not(self &~ other)

    def __ge__(self,other):
        return not(other &~ self)

    def __lt__(self,other):
        return self <= other and self != other

    def __gt__(self,other):
        return self >= other and self != other

    def __invert__(self):
        """Complement (with respect to alphabet) of language."""
        return Language(self.recognizer.complement())

    def __and__(self,other):
        """Intersection of two languages with the same alphabet."""
        if not isinstance(other,RegularLanguage):
            raise LanguageError("Unable to intersect nonregular language")
        return Language(self.recognizer.intersection(other.recognizer))

    def __or__(self,other):
        """Union of two languages with the same alphabet."""
        if not isinstance(other,RegularLanguage):
            raise LanguageError("Unable to intersect nonregular language")
        return Language(self.recognizer.union(other.recognizer))

    def __xor__(self,other):
        """Symmetric difference of two languages with the same alphabet."""
        if not isinstance(other,RegularLanguage):
            raise LanguageError("Unable to intersect nonregular language")
        return Language(self.recognizer.symmetricDifference(other.recognizer))

    def __nonzero__(self):
        """Is this the empty language?"""
        for x in self.recognizer.states():
            if x.isfinal():
                return True
        return False

class FiniteAutomaton:
    """Base class for DFA and NFA.  This class should not be instantiated
    on its own, but dispatches methods that are appropriate to both types
    of automaton by calling .asDFA() or .asNFA() to convert the automaton
    to the appropriate type.  All automaton instances should include the
    following instance variables and methods:
     - x.initial: initial state (for DFA) or set of states (for NFA)
     - x.alphabet: set of input symbols accepted by the automaton
     - x.transition(state,symbol): result of transition function,
       either a single state (for a DFA) or set of states (for an NFA)
     - x.isfinal(state): whether the state is an accepting state
     - x.asDFA(): return an equivalent DFA
     - x.asNFA(): return an equivalent NFA
    """

    initial = alphabet = transition = isfinal = asDFA = asNFA = None

    def __len__(self):
        """How many states does this automaton have?"""
        return len(list(self.states()))

    def __call__(self,symbols):
        """Test whether sequence of symbols is accepted by the DFA."""
        return self.asDFA()(symbols)

    def language(self):
        """Form language object for language recognized by automaton."""
        return RegularLanguage(self)

    def states(self):
        """Generate all states reachable from initial state."""
        return self.asNFA().states()

    def pprint(self,output=sys.stdout):
        """Pretty-print this automaton to an output stream."""
        return self.asNFA().pprint(output)

    def to_graph(self, symbol_table):
        """Pretty-print this automaton to an output stream."""
        return self.asNFA().to_graph(symbol_table)

    def minimize(self):
        """Return smallest equivalent DFA."""
        return _MinimumDFA(self.asDFA())

    def reverse(self):
        """Construct NFA for reversal of original NFA's language."""
        return _ReverseNFA(self.asNFA())

    def renumber(self,offset=0):
        """Replace complicated state objects by small integers."""
        return _RenumberNFA(self.asNFA(),offset=offset)

    def RegExp(self):
        """Return equivalent regular expression."""
        return self.asNFA().RegExp()

    def complement(self):
        """Make automaton recognizing complement of given automaton's language."""
        return _ComplementDFA(self.asDFA())

    def union(self,other):
        """Make automaton recognizing union of two automata's languages."""
        return _ProductDFA(self.asDFA(),other.asDFA(),operator.or_)

    def intersection(self,other):
        """Make automaton recognizing union of two automata's languages."""
        return _ProductDFA(self.asDFA(),other.asDFA(),operator.and_)

    def symmetricDifference(self,other):
        """Make automaton recognizing union of two automata's languages."""
        return _ProductDFA(self.asDFA(),other.asDFA(),operator.xor)

class DFA(FiniteAutomaton):
    """Base class for deterministic finite automaton.  Subclasses are
    responsible for filling out the details of the initial state, alphabet,
    and transition function.
    """
    def asDFA(self):
        return self

    def asNFA(self):
        return _NFAfromDFA(self)

    def __call__(self,symbols):
        """Test whether sequence of symbols is accepted by the DFA."""
        state = self.initial
        for symbol in symbols:
            if symbol not in self.alphabet:
                raise LanguageError("Symbol " + repr(symbol) +
                                 " not in input alphabet")
            state = self.transition(state,symbol)
        return self.isfinal(state)

    def __eq__(self,other):
        """Report whether these two DFAs have equivalent states."""
        if not isinstance(other,DFA) or len(self) != len(other) \
                or self.alphabet != other.alphabet:
            return False
        equivalences = {self.initial:other.initial}
        unprocessed = [self.initial]
        while unprocessed:
            x = unprocessed.pop()
            y = equivalences[x]
            for c in self.alphabet:
                xc = self.transition(x,c)
                yc = other.transition(y,c)
                if xc not in equivalences:
                    equivalences[xc] = yc
                    unprocessed.append(xc)
                elif equivalences[xc] != yc:
                    return False
        return True

    def __ne__(self,other):
        """Report whether these two DFAs have equivalent states."""
        return not (self == other)

class NFA(FiniteAutomaton):
    """Base class for nondeterministic finite automaton.  Subclasses are
    responsible for filling out the details of the initial state, alphabet,
    and transition function.  Note that the NFAs defined here do not allow
    epsilon-transitions.  Results of self.initial and self.transition are
    assumed to be represented as frozenset instances.
    """
    def asNFA(self):
        return self

    def asDFA(self):
        return _DFAfromNFA(self)

    def states(self):
        visited = set()
        unvisited = set(self.initial)
        while unvisited:
            state = arbitrary_item(unvisited)
            yield state
            unvisited.remove(state)
            visited.add(state)
            for symbol in self.alphabet:
                unvisited |= self.transition(state,symbol) - visited

    def to_graph(self, symbol_table):
        if symbol_table is None:
            return 0
        g = Graph()
        """Creates a graph out of the automaton."""
        for state in self.states():
            accepting = False
            initial = False
            adjectives = []
            if state in self.initial:
                adjectives.append("initial")
                initial = True
            if self.isfinal(state):
                adjectives.append("accepting")
                accepting = True
            if not [c for c in self.alphabet if self.transition(state,c)]:
                adjectives.append("terminal")
            if not adjectives:
                pass
            else:
                pass
            for c in self.alphabet:
                for neighbor in self.transition(state,c):
                    aps = symbol_table['letter'][str(c)]
                    region = ''
                    for ap in list(aps):
                        if isinstance(region,str):
                            region =  symbol_table['ap'][ap]
                        else:
                            region &= symbol_table['ap'][ap]
                    g.add(state, region, neighbor, accepting, initial)
        return g
                
    def pprint(self,output=sys.stdout):
        """Pretty-print this NFA to an output stream."""
        for state in self.states():
            adjectives = []
            if state in self.initial:
                adjectives.append("initial")
            if self.isfinal(state):
                adjectives.append("accepting")
            if not [c for c in self.alphabet if self.transition(state,c)]:
                adjectives.append("terminal")
            if not adjectives:
                print >>output, state
            else:
                print >>output, state, "(" + ", ".join(adjectives) + ")"
            for c in self.alphabet:
                for neighbor in self.transition(state,c):
                    print >>output, "  --[" + str(c) + "]-->", neighbor

    def RegExp(self):
        """Convert to regular expression and return as a string.
        See Sipser for an explanation of this algorithm."""

        # create artificial initial and final states
        initial = object()
        final = object()
        states = {initial,final} | set(self.states())

        # 2d matrix of expressions connecting each pair of states
        expr = {}
        for x in states:
            for y in states:
                expr[x,y] = None
        for x in self.states():
            if x in self.initial:
                expr[initial,x] = ''
            if self.isfinal(x):
                expr[x,final] = ''
            expr[x,x] = ''
        for x in self.states():
            for c in self.alphabet:
                for y in self.transition(x,c):
                    if expr[x,y]:
                        expr[x,y] += '+' + str(c)
                    else:
                        expr[x,y] = str(c)

        # eliminate states one at a time
        for s in self.states():
            states.remove(s)
            for x in states:
                for y in states:
                    if expr[x,s] is not None and expr[s,y] is not None:
                        xsy = []
                        if expr[x,s]:
                            xsy += self._parenthesize(expr[x,s])
                        if expr[s,s]:
                            xsy += self._parenthesize(expr[s,s],True) + ['*']
                        if expr[s,y]:
                            xsy += self._parenthesize(expr[s,y])
                        if expr[x,y] is not None:
                            xsy += ['+',expr[x,y] or '()']
                        expr[x,y] = ''.join(xsy)
        return expr[initial,final]

    def _parenthesize(self,expr,starring=False):
        """Return list of strings with or without parens for use in RegExp.
        This is only for the purpose of simplifying the expressions returned,
        by omitting parentheses or other expression features when unnecessary;
        it would always be correct simply to return ['(',expr,')'].
        """
        if len(expr) == 1 or (not starring and '+' not in expr):
            return [expr]
        elif starring and expr.endswith('+()'):
            return ['(',expr[:-3],')']  # +epsilon redundant when starring
        else:
            return ['(',expr,')']

class _DFAfromNFA(DFA):
    """Conversion of NFA to DFA.  We create a DFA state for each set
    of NFA states. A DFA state is final if it contains at least one
    final NFA set, and the transition function for a DFA state is the
    union of the transition functions of the NFA states it contains.
    """
    def __init__(self,N):
        self.initial = frozenset(N.initial)
        self.alphabet = N.alphabet
        self.NFA = N
        self.symbol_table = N.symbol_table

    def transition(self,stateset,symbol):
        result = set()
        for state in stateset:
            result |= self.NFA.transition(state,symbol)
        return frozenset(result)

    def isfinal(self,stateset):
        for state in stateset:
            if self.NFA.isfinal(state):
                return True
        return False

class _NFAfromDFA(NFA):
    """Conversion of DFA to NFA.  We convert the initial state and the
    results of each transition function into single-element sets.
    """
    def __init__(self,D):
        self.initial = frozenset([D.initial])
        self.alphabet = D.alphabet
        self.DFA = D
        self.symbol_table = D.symbol_table

    def transition(self,state,symbol):
        return frozenset([self.DFA.transition(state,symbol)])

    def isfinal(self,state):
        return self.DFA.isfinal(state)

class RegExp(NFA):
    """Convert regular expression to NFA."""

    def __init__(self,expr, symbol_table=None):
        self.expr = expr
        self.pos = 0
        self.nstates = 0
        self.expect = {}
        self.successor = {}
        self.alphabet = set()
        self.initial,penultimate,epsilon = self.expression()
        self.symbol_table = symbol_table
        final = self.newstate(None)
        for state in penultimate:
            self.successor[state].add(final)
        self.final = frozenset([final])
        if epsilon:
            self.final = self.final | self.initial

    def transition(self,state,c):
        """Implement NFA transition function."""
        if c != self.expect[state]:
            return frozenset()
        else:
            return self.successor[state]

    def isfinal(self,state):
        """Implement NFA acceptance test."""
        return state in self.final

    # Recursive-descent parser for regular expressions.
    # Each function uses self.pos as a pointer into self.expr,
    # updates self.expect and self.successor,
    # and returns a tuple (initial,penultimate,epsilon), where
    #   initial = the initial states of the subexpression
    #   penultimate = states one step away from an accepting state
    #   epsilon = true if the subexpression accepts the empty string

    def epsilon(self):
        """Parse an empty string and return an empty automaton."""
        return frozenset(),frozenset(),True

    def newstate(self,expect):
        """Allocate a new state in which we expect to see the given letter."""
        state = self.nstates
        self.successor[state] = set()
        self.expect[state] = expect
        self.nstates += 1
        return state

    def base(self):
        """Parse a subexpression that can be starred: single letter or group."""
        if self.pos == len(self.expr) or self.expr[self.pos] == ')':
            return self.epsilon()
        if self.expr[self.pos] == '(':
            self.pos += 1
            ret = self.expression()
            if self.pos == len(self.expr) or self.expr[self.pos] != ')':
                raise LanguageError("Close paren expected at char " + str(self.pos))
            self.pos += 1
            return ret
        if self.expr[self.pos] == '\\':
            self.pos += 1
            if self.pos == len(self.expr):
                raise RegExpError("Character expected after backslash")
        self.alphabet.add(self.expr[self.pos])
        state = self.newstate(self.expr[self.pos])
        self.pos += 1
        state = frozenset([state])
        return state,state,False

    def factor(self):
        """Parse a catenable expression: base or starred base."""
        initial,penultimate,epsilon = self.base()
        while self.pos < len(self.expr) and self.expr[self.pos] == '*':
            self.pos += 1
            for state in penultimate:
                self.successor[state] |= initial
            epsilon = True
        return initial,penultimate,epsilon

    def term(self):
        """Parse a summable expression: factor or concatenation."""
        initial,penultimate,epsilon = self.factor()
        while self.pos < len(self.expr) and self.expr[self.pos] not in ')+':
            Fi,Fp,Fe = self.factor()
            for state in penultimate:
                self.successor[state] |= Fi
            if epsilon:
                initial = initial | Fi
            if Fe:
                penultimate = penultimate | Fp
            else:
                penultimate = Fp
            epsilon = epsilon and Fe
        return initial,penultimate,epsilon

    def expression(self):
        """Parse a whole regular expression or grouped subexpression."""
        initial,penultimate,epsilon = self.term()
        while self.pos < len(self.expr) and self.expr[self.pos] == '+':
            self.pos += 1
            Ti,Tp,Te = self.term()
            initial = initial | Ti
            penultimate = penultimate | Tp
            epsilon = epsilon or Te
        return initial,penultimate,epsilon

class LookupNFA(NFA):
    """Construct NFA with precomputed lookup table of transitions."""
    def __init__(self,alphabet,initial,ttable,final):
        self.alphabet = alphabet
        self.initial = frozenset(initial)
        self.ttable = ttable
        self.final = frozenset(final)

    def transition(self,state,symbol):
        return frozenset(self.ttable[state,symbol])

    def isfinal(self,state):
        return state in self.final

def _RenumberNFA(N,offset=0):
    """Replace NFA state objects with small integers."""
    replacements = {}
    for x in N.states():
        replacements[x] = offset
        offset += 1
    initial = [replacements[x] for x in N.initial]
    ttable = {}
    for state in N.states():
        for symbol in N.alphabet:
            ttable[replacements[state],symbol] = [replacements[x]
                for x in N.transition(state,symbol)]
    final = [replacements[x] for x in N.states() if N.isfinal(x)]
    return LookupNFA(N.alphabet,initial,ttable,final)

class _ProductDFA(DFA):
    """DFA that simulates D1 and D2 and combines their outputs with op."""
    def __init__(self,D1,D2,op):
        if D1.alphabet != D2.alphabet:
            raise LanguageError("DFAs have incompatible alphabets")
        self.alphabet = D1.alphabet
        self.initial = (D1.initial,D2.initial)
        self.D1 = D1
        self.D2 = D2
        self.op = op

    def transition(self,state,symbol):
        s1,s2 = state
        return self.D1.transition(s1,symbol), \
               self.D2.transition(s2,symbol)

    def isfinal(self,state):
        s1,s2 = state
        f1 = self.D1.isfinal(s1) and 1 or 0
        f2 = self.D2.isfinal(s2) and 1 or 0
        return self.op(f1,f2)

def _ReverseNFA(N):
    """Construct NFA for reversal of original NFA's language."""
    initial = [s for s in N.states() if N.isfinal(s)]
    ttable = {(s,c):[] for s in N.states() for c in N.alphabet}
    for s in N.states():
        for c in N.alphabet:
            for t in N.transition(s,c):
                ttable[t,c].append(s)
    return LookupNFA(N.alphabet,initial,ttable,N.initial)

class _ComplementDFA(DFA):
    """DFA for complementary language."""
    def __init__(self,D):
        self.DFA = D
        self.initial = D.initial
        self.alphabet = D.alphabet

    def transition(self,state,symbol):
        return self.DFA.transition(state,symbol)

    def isfinal(self,state):
        return not self.DFA.isfinal(state)

class _MinimumDFA(DFA):
    """Construct equivalent DFA with minimum number of states,
    using Hopcroft's O(ns log n) partition-refinement algorithm.
    """
    def __init__(self,D):
        # refine partition of states by reversed neighborhoods
        N = D.reverse()
        P = PartitionRefinement(D.states())
        P.refine([s for s in D.states() if D.isfinal(s)])
        unrefined = Sequence(P,key=id)
        while unrefined:
            part = arbitrary_item(unrefined)
            unrefined.remove(part)
            for symbol in D.alphabet:
                neighbors = set()
                for state in part:
                    neighbors |= N.transition(state,symbol)
                for new,old in P.refine(neighbors):
                    if old in unrefined or len(new) < len(old):
                        unrefined.append(new)
                    else:
                        unrefined.append(old)

        # convert partition to DFA
        P.freeze()
        self.partition = P
        self.initial = P[D.initial]
        self.alphabet = D.alphabet
        self.DFA = D

    def transition(self,state,symbol):
        rep = arbitrary_item(state)
        return self.partition[self.DFA.transition(rep,symbol)]

    def isfinal(self,state):
        rep = arbitrary_item(state)
        return self.DFA.isfinal(rep)

# If called as standalone routine, run some unit tests

#class RegExpTest(unittest.TestCase):
#    # tuples (L,[strings in L],[strings not in L])
#    languages = [
#        (RegularLanguage("0"), ["0"], ["","00"]),
#        (RegularLanguage("(10+0)*"), ["","0","010"], ["1"]),
#        (RegularLanguage("(0+1)*1(0+1)(0+1)"), ["000100"], ["0011"]),
#    ]

#    def testMembership(self):
#        """membership tests for RegularLanguage(expression)"""
#        for L,Li,Lx in self.languages:
#            for S in Li:
#                self.assertTrue(S in L)
#            for S in Lx:
#                self.assertTrue(S not in L)

#    def testComplement(self):
#        """membership tests for ~RegularLanguage"""
#        for L,Li,Lx in self.languages:
#            L = ~L
#            for S in Lx:
#                self.assertTrue(S in L)
#            for S in Li:
#                self.assertTrue(S not in L)

#    def testEquivalent(self):
#        """test that converting NFA->expr->NFA produces same language"""
#        for L1,Li,Lx in self.languages:
#            L2 = RegularLanguage(L1.recognizer.RegExp())
#            self.assertEqual(L1,L2)

#    def testInequivalent(self):
#        """test that different regular languages are recognized as different"""
#        for i in range(len(self.languages)):
#            for j in range(i):
#                self.assertNotEqual(self.languages[i][0],
#                                    self.languages[j][0])

#if __name__ == "__main__":
#    unittest.main()   
